﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace dcmtoblock {
    /// <summary>
    /// A 3D contour, containing all the contours composing a particular masked organ
    /// </summary>
    public class Contour {
        public string Name { get; set; }
        public string SeriesUID { get; set; }
        public ContourPlane[] ContourPlanes { get; set; }

        /// <summary>
        /// Given the collection of image planes associated with the contour,
        /// generate a 3D datablock of the same dimensions with a 1 if the pixel is in the contour
        /// and zero otherwise.
        /// </summary>
        /// <returns></returns>
        public ContourMask3D Produce3DContourMask(Image3DBlock inBlock) {

            ContourMask3D theReturn = new ContourMask3D();
            theReturn.Mask = new byte[inBlock.XSize * inBlock.YSize * inBlock.ZSize];


            theReturn.Name = Name;
            theReturn.SeriesUID = SeriesUID;
            theReturn.XSize = inBlock.XSize;
            theReturn.YSize = inBlock.YSize;
            theReturn.ZSize = inBlock.ZSize;

            //not so simple as this loop-- have to make sure that I've got the coordinate transforms
            //done as well
            int zloc = 0;//z needs to be set to the proper start plane, somehow.
            if (ContourPlanes == null) return theReturn; //ie, if the contours weren't read in properly.
            foreach (ContourPlane plane in ContourPlanes) {
                if (plane.Verteces == null) continue;//no verteces, can't convert to 3D mask



                //have to look through all the planes in the image block to find the one with the right z value
                //contours are not necessarily in order, it seems
                for (int z = 0; z < inBlock.ZSize; z++) {
                    if (Math.Abs(inBlock.Planes[z].UpperLeft.Z - plane.Verteces[0].Z) < 0.001) {//because sometimes there's an extra digit
                        zloc = z;
                        break;
                    }
                }
                if (Name.ToLower().Contains("liver")) {
                    zloc = zloc;//use this statement to set a breakpoint for debugging
                }
                //we need to know the plane in order to get the coordinate transforms.
                //however, we don't necessarily need to know where the plane is in the z stack, just the x-y coords.
                byte[] theMaskPlane = plane.ProduceContourMask(inBlock.Planes[zloc]);

                //instead of doing a block copy, do a point-by-point addition.  That way, there can be multiple closed masks per plane

                //System.Buffer.BlockCopy(theMaskPlane, 0, theReturn.Mask, 
                //    zloc * theReturn.XSize * theReturn.YSize, theReturn.XSize * theReturn.YSize * sizeof(byte));
                int x, y;
                for (y = 0; y < theReturn.YSize; ++y) {
                    for (x = 0; x < theReturn.XSize; ++x) {//do the xor, to remove masks that are not supposed to be present
                        theReturn.Mask[zloc*theReturn.YSize*theReturn.XSize + y * theReturn.XSize + x] ^= theMaskPlane[y * theReturn.XSize + x];
                    }
                }

            }

            return theReturn;

        }
    }
}
