﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;

namespace dcmtoblock {
    /// <summary>
    /// the actual mask itself; the byte array is xsize * ysize * zsize
    /// This class is intended to be read out by protobuffer and read in by the same mechanism.
    /// but for now, we'll just write it out as a text file, because I'm super lazy.
    /// </summary>
    public class ContourMask3D {

        public ContourMask3D() { }//for protobuf

        public byte[] Mask { get; set; }//has to be a bool to be read out by protobufs
        
        public int XSize { get; set; }

        public int YSize { get; set; }

        public int ZSize { get; set; }
        
        public string Name { get; set; }
        
        public string SeriesUID { get; set; }

        private static string MakeValidFileName(string name) {
            string invalidChars = Regex.Escape(new string(Path.GetInvalidFileNameChars()));
            string invalidReStr = string.Format(@"[{0}]", invalidChars);
            return Regex.Replace(name, invalidReStr, "_");
        }

        public void WriteToDisk(string theDirectory, string inUID) {
            using (FileStream fs = new FileStream(theDirectory + "\\" +
                MakeValidFileName(Name) + "." + inUID + ".byteblock", FileMode.Create)) {
                //ProtoBuf.Serializer.Serialize<Image3DBlock>(fs, theBlock);

                byte[] theBuffer = System.BitConverter.GetBytes(XSize);
                fs.Write(theBuffer, 0, theBuffer.Length);
                theBuffer = System.BitConverter.GetBytes(YSize);
                fs.Write(theBuffer, 0, theBuffer.Length);
                theBuffer = System.BitConverter.GetBytes(ZSize);
                fs.Write(theBuffer, 0, theBuffer.Length);

                theBuffer = new byte[Mask.Length * sizeof(byte)];
                System.Buffer.BlockCopy(Mask, 0, theBuffer, 0, Mask.Length * sizeof(byte));
                fs.Write(theBuffer, 0, theBuffer.Length);
            }
        }
    }
}
