﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace dcmtoblock {
    
    ///this class will define a contour for a particular organ for a particular study for a particular plane
    ///it will contain the list of points stored in the RS file, and has a routine
    ///to translate those points into a 2D plane of byte data (on or off, depending on inclusion/exclusion
    public class ContourPlane {
        /// <summary>
        /// probably the organ name, at least it is in the initial data
        /// </summary>

        public DicomVertex[] Verteces { get; set; }

        //the value set for a point in the contour definition
        const int SpecifiedCoordinate = 20;
        const double SubsampleLine = 10.0;//increase this amount, just in case the image resizing makes a vastly larger image.

        ///make sure you have the right plane!
        ///This function will ignore z, once it has the plane,and concentrate on making
        ///planar data from the vertex collection it should have.
        public byte[] ProduceContourMask(ImagePlane inPlane) {

            byte[] theReturn = new byte[inPlane.XSize * inPlane.YSize];
            
            int y, x, k;
            //now, let's transform the coordinates, and place them in the appropriate spots
            for (k = 0; k < Verteces.Length; k++){
                //have to draw a line connect the current point with the next point
                //transform the x/y coordinates
                double xcoord = ((Verteces[k].X - inPlane.UpperLeft.X) / inPlane.XSpacing);
                double ycoord = ((Verteces[k].Y - inPlane.UpperLeft.Y) / inPlane.YSpacing);

                double xcoordNext, ycoordNext;
                if (k == Verteces.Length - 1) {
                    xcoordNext = ((Verteces[0].X - inPlane.UpperLeft.X) / inPlane.XSpacing);
                    ycoordNext = ((Verteces[0].Y - inPlane.UpperLeft.Y) / inPlane.YSpacing);
                }
                else {
                    xcoordNext = ((Verteces[k+1].X - inPlane.UpperLeft.X) / inPlane.XSpacing);
                    ycoordNext = ((Verteces[k+1].Y - inPlane.UpperLeft.Y) / inPlane.YSpacing);
                }
                //subsample along the line 3 times, just to make sure it's a closed polygon
                //still have to do a cast to make it fit, which could be unfortunate.

                for (y = 0; y <= SubsampleLine; y++) {
                    double xcurr = y * ((xcoordNext - xcoord) / SubsampleLine) + xcoord;
                    double ycurr = y * ((ycoordNext - ycoord) / SubsampleLine) + ycoord;

                    theReturn[(int)(ycurr) * inPlane.XSize + (int)(xcurr)] = SpecifiedCoordinate; //some large number

                }

            }

            

            //now, loop through the return, one row at a time.
            int ysize = inPlane.YSize;
            int xsize = inPlane.XSize;//most likely 512x512 anyway, but just in case.

            byte[] theSecondReturn = new byte[theReturn.Length];
            System.Buffer.BlockCopy(theReturn, 0, theSecondReturn, 0, theReturn.Length * sizeof(byte));

            for (y = 0; y < ysize; y++) {
                List<int> points = new List<int>();
                for (x = 1; x < xsize; x++) {//assume no masks near the edges of the CT image
                    if (theReturn[y * xsize + x] != SpecifiedCoordinate &&
                        theReturn[y * xsize + x - 1] == SpecifiedCoordinate) {
                        points.Add(x);
                    }
                }
                //now, go from point to point, and determine if the previous line had anything interesting.
                //if so, mark away!
                for(x = 0; x < points.Count -1; x++){
                    bool extraLine = false;
                    if (y != 0) {//lines on top are marked, period.
                        for (k = points[x]; k < points[x + 1]; k++) {
                            //if the current line isn't designated by the verteces, and
                            //the line above is unmarked, then this line will be unmarked.
                            if (theReturn[y * xsize + k] != SpecifiedCoordinate) {
                                if (theReturn[(y - 1) * xsize + k] < 1) {
                                    extraLine = true;
                                    break;
                                }
                            }
                        }
                    }
                    if (!extraLine) {
                        //the previous line was marked, so this will be marked.
                        for (k = points[x]; k < points[x+1]; k++) {
                            if (theReturn[y * xsize + k] < SpecifiedCoordinate)
                                theReturn[y * xsize + k] += (byte) 1;
                        }
                    }
                }
            }

            //now, run it from the bottom up, on theSecondReturn 
            for (y = ysize-1; y > 0; y--) {
                List<int> points = new List<int>();
                for (x = 1; x < xsize; x++) {//assume no masks near the edges of the CT image
                    if (theSecondReturn[y * xsize + x] != SpecifiedCoordinate &&
                        theSecondReturn[y * xsize + x - 1] == SpecifiedCoordinate) {
                        points.Add(x);
                    }
                }
                //now, go from point to point, and determine if the previous line had anything interesting.
                //if so, mark away!
                for(x = 0; x < points.Count -1; x++){
                    bool extraLine = false;
                    if (y != ysize - 1) {//lines on bottom get marked.
                        for (k = points[x]; k < points[x + 1]; k++) {
                            //if the current line isn't designated by the verteces, and
                            //the line above is unmarked, then this line will be unmarked.

                            if (theSecondReturn[y * xsize + k] != SpecifiedCoordinate) {
                                if (theSecondReturn[(y + 1) * xsize + k] < 1) {
                                    extraLine = true;
                                    break;
                                }
                            }
                        }
                    }
                    if (!extraLine) {
                        //the previous line was marked, so this will be marked.
                        for (k = points[x]; k < points[x+1]; k++) {
                            if (theSecondReturn[y * xsize + k] < SpecifiedCoordinate)
                                theSecondReturn[y * xsize + k] += (byte)1;
                        }
                    }
                }
            }
                

            //now, just keep the border pixels or the ones that have been marked 4 times.
            for (y = 0; y < ysize; y++) {
                for (x = 0; x < xsize; x++) {
                    if ((theReturn[y * xsize + x] == SpecifiedCoordinate) || 
                        (theReturn[y * xsize + x] + theSecondReturn[y * xsize + x] == 2)) {
                        theReturn[y * xsize + x] = 1;
                    }
                    else {
                        theReturn[y * xsize + x] = 0;
                    }
                }
            }
            

            return theReturn;
        }
    }
}
