﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using gdcm;

namespace dcmtoblock {
    /// <summary>
    /// reads an individual dicom plane and returns that plane.
    /// </summary>
    public class DicomReader {
        /// <summary>
        /// Reads a dicom file into an ImagePlane object
        /// </summary>
        /// <param name="inFileName"></param>
        /// <returns></returns>
        public static ImagePlane ReadDicomImage(string inFileName, double inSubsampleAmount) {
            ImagePlane theReturn = new ImagePlane();

            gdcm.ImageReader theReader = new gdcm.ImageReader();
            theReader.SetFileName(inFileName);
            if (!theReader.Read()) {
                theReader.Dispose();
                return null;
            }
            gdcm.Image theImage = theReader.GetImage();
            theReturn.XSize = (int)theImage.GetColumns();
            theReturn.YSize = (int)theImage.GetRows();
            byte[] theBuffer = new byte[theImage.GetBufferLength()];
            if (!theImage.GetBuffer(theBuffer)) {
                theReader.Dispose();
                return null;
            }
            theReturn.Data = new short[theReturn.XSize * theReturn.YSize];
            theReturn.XSpacing = theImage.GetSpacing(0);
            theReturn.YSpacing = theImage.GetSpacing(1);

            DataSet theDataSet = theReader.GetFile().GetDataSet();
            DataElement zthickness = theDataSet.GetDataElement(new Tag(0x0018, 0x0050));

            theReturn.ZSpacing = float.Parse(GetStringValueFromTag(zthickness));//do these files define the zspacing? I don't know...

            theReturn.UpperLeft = GetVertexValuesFromTag(theDataSet.GetDataElement(new Tag(0x0020,0x0032)));

            //WriteOutAllDataSetTags(theDataSet, inFileName);

            //series uid is a definite requirement, it's a type 1 tag.  So the following code must work,
            ///andif it doesn't, then it's a malformed DICOM image
            theReturn.SeriesUID = GetStringValueFromTag(theDataSet.GetDataElement(new Tag(0x0020, 0x000E)));

            if (!theImage.GetArray(theReturn.Data)) {
                theReader.Dispose();
                return null;
            }
            theReader.Dispose();

            if (inSubsampleAmount > 0) {
                theReturn.Subsample(inSubsampleAmount);
            }
            return theReturn;
        }

        public static Contour[] ReadRSStruct(string inFileName)
        {
            gdcm.Reader theReader = new gdcm.Reader();
            theReader.SetFileName(inFileName);
            theReader.Read();//make sure to read the file...
            gdcm.File theFile = theReader.GetFile();
            //for the data we have (june 2009), that previous statement is true
            //which means that there's just a contour sequence, not a set of sequences
            //so the straight gdcm code will fail.
            //that means it's just a straight block of voxels!
            //not a proper dicom RS Sequence!
            //so we go directly to reading from tag (3006, 0050);

            // (3006,0050) DS [43.57636\65.52504\-10.0\46.043102\62.564945\-10.0\49.126537\60.714... # 398,48 ContourData
            DataSet theDataSet = theFile.GetDataSet();

           /* List<Tag> theTags = new List<Tag>();
            theTags.Add(new Tag(0x3006, 0x0002));//name, type 1
            theTags.Add(new Tag(0x3006, 0x0004));
            theTags.Add(new Tag(0x3006, 0x0006));
            theTags.Add(new Tag(0x3006, 0x0008));
            theTags.Add(new Tag(0x3006, 0x0009));

            theTags.Add(new Tag(0x3006, 0x0010));
            theTags.Add(new Tag(0x3006, 0x0012));
            theTags.Add(new Tag(0x3006, 0x0014)); //type 1
            theTags.Add(new Tag(0x3006, 0x0016)); //type 1

            theTags.Add(new Tag(0x3006, 0x0020));
            theTags.Add(new Tag(0x3006, 0x0022)); // type 1
            theTags.Add(new Tag(0x3006, 0x0024)); //type 1
            theTags.Add(new Tag(0x3006, 0x0026));
            theTags.Add(new Tag(0x3006, 0x0028));
            theTags.Add(new Tag(0x3006, 0x002A));
            theTags.Add(new Tag(0x3006, 0x002C));


            theTags.Add(new Tag(0x3006, 0x0030));
            theTags.Add(new Tag(0x3006, 0x0033));
            theTags.Add(new Tag(0x3006, 0x0036));
            theTags.Add(new Tag(0x3006, 0x0038));
            theTags.Add(new Tag(0x3006, 0x0039)); //type 1

            theTags.Add(new Tag(0x3006, 0x0040));
            theTags.Add(new Tag(0x3006, 0x0042)); //type 1
            theTags.Add(new Tag(0x3006, 0x0044));
            theTags.Add(new Tag(0x3006, 0x0045));
            theTags.Add(new Tag(0x3006, 0x0046));//type 1
            theTags.Add(new Tag(0x3006, 0x0048));
            theTags.Add(new Tag(0x3006, 0x0049));

            theTags.Add(new Tag(0x3006, 0x0050)); //type 1 (if closed plane, then all are coplanar)

            theTags.Add(new Tag(0x3006, 0x0080));
            theTags.Add(new Tag(0x3006, 0x0082));
            theTags.Add(new Tag(0x3006, 0x0084)); //type 1
            theTags.Add(new Tag(0x3006, 0x0085));
            theTags.Add(new Tag(0x3006, 0x0086));
            theTags.Add(new Tag(0x3006, 0x0088));

            theTags.Add(new Tag(0x3006, 0x00A0));
            theTags.Add(new Tag(0x3006, 0x00A4));
            theTags.Add(new Tag(0x3006, 0x00A6));

            theTags.Add(new Tag(0x3006, 0x00B0));
            theTags.Add(new Tag(0x3006, 0x00B2));
            theTags.Add(new Tag(0x3006, 0x00B4));
            theTags.Add(new Tag(0x3006, 0x00B6));
            theTags.Add(new Tag(0x3006, 0x00B7));
            theTags.Add(new Tag(0x3006, 0x00B8));

            theTags.Add(new Tag(0x3006, 0x00C0));
            theTags.Add(new Tag(0x3006, 0x00C2));
            theTags.Add(new Tag(0x3006, 0x00C4));
            theTags.Add(new Tag(0x3006, 0x00C6));//type 1
            theTags.Add(new Tag(0x3006, 0x00C8));

            theTags.Add(new Tag(0x0020, 0x0052));//frame of reference, type 1
            theTags.Add(new Tag(0x0020, 0x000E));//series UID, type 1


            bool foundSomething = false;
            foreach (Tag t in theTags) {
                if (theDataSet.FindDataElement(t)) {
                    foundSomething = true;
                    string theTagString = t.toString();
                    System.Console.WriteLine("Found tag! " + theTagString.Substring(0, (theTagString.Length > 100 ? 100 : theTagString.Length)));
                }
            }
            if (!foundSomething) {
                System.Console.WriteLine("No tags were found, sorry.");
                theReader.Dispose();
                return null; 
            }
            */
            //WriteOutAllDataSetTags(theDataSet, inFileName);
            //find the StructureSetROISequence
            // (3006,0020) SQ (Sequence with explicit length #=4)      # 370, 1 StructureSetROISequence  
            // (3006,0039) SQ (Sequence with explicit length #=4)      # 24216, 1 ROIContourSequence
            gdcm.Tag theROIContourSequenceTag = new Tag(0x3006, 0x0039);
            gdcm.Tag theStructureSetROISequenceTag = new Tag(0x3006, 0x0020);
            if (!theDataSet.FindDataElement(theROIContourSequenceTag) || 
                !theDataSet.FindDataElement(theStructureSetROISequenceTag)){
            
                theReader.Dispose();
                return null;
            }

            gdcm.DataElement theROIContourSequenceDE =
                theDataSet.GetDataElement(theROIContourSequenceTag);

            gdcm.SmartPtrSQ theROIContourSequence = 
                theROIContourSequenceDE.GetValueAsSQ();

            if (theROIContourSequence == null || 
                theROIContourSequence.GetNumberOfItems() == 0) {
                theReader.Dispose();
                return null;
            }
            
            gdcm.DataElement theStructureSetROISequenceTagDE = 
                theDataSet.GetDataElement(theStructureSetROISequenceTag);

            gdcm.SmartPtrSQ theStructureSetROISequence = 
                theStructureSetROISequenceTagDE.GetValueAsSQ();
            if (theStructureSetROISequence == null ||
                theStructureSetROISequence.GetNumberOfItems() == 0) {
                theReader.Dispose();
                return null;
            }

            char[] splitters = new char[1];
            splitters[0] = '\\';//the numbers are split by the escape sequence
            uint pd;
            uint theROIContourSequenceCount = theROIContourSequence.GetNumberOfItems();
            Contour[] theReturn = new Contour[theROIContourSequenceCount];
            for (pd = 0; pd < theROIContourSequenceCount; ++pd) {//not sure why the ++pd instead of pd++
                gdcm.Item item = theROIContourSequence.GetItem(pd + 1); //items start at 1, not 0
                gdcm.Item sitem = theStructureSetROISequence.GetItem(pd + 1);
                gdcm.DataSet snestedds = sitem.GetNestedDataSet();
                // (3006,0026) ?? (LO) [date]                                    # 4,1 ROI Name
                gdcm.Tag stcsq = new Tag(0x3006, 0x0026);
                if (!snestedds.FindDataElement(stcsq)) {//guess each ROI needs a name
                    continue; //this contoure is busted
                }

                DataElement sde = snestedds.GetDataElement(stcsq);
                theReturn[pd] = new Contour();
                theReturn[pd].Name = GetStringValueFromTag(sde);
                theReturn[pd].SeriesUID = GetStringValueFromTag(theDataSet.GetDataElement(new Tag(0x0020, 0x000E)));

                DataSet nestedds = item.GetNestedDataSet();

                //(3006,002a) IS [255\192\96]                              # 10,3 ROI Display Color
                Tag troidc = new Tag(0x3006, 0x002a);
                //we don't care about color, skipping.
                //(3006,0040) SQ (Sequence with explicit length #=8)      # 4326, 1 ContourSequence
                Tag tcsq = new Tag(0x3006, 0x0040);
                if (!nestedds.FindDataElement(tcsq)) {
                    continue; //this contoure is busted
                //    theReader.Dispose();
                //    return null;
                }
                DataElement csq = nestedds.GetDataElement(tcsq);
                gdcm.SmartPtrSQ sqi2 = csq.GetValueAsSQ();
                if (sqi2 == null || sqi2.GetNumberOfItems() == 0) {
                    continue; //this contoure is busted
                }
                uint nitems = sqi2.GetNumberOfItems();

                theReturn[pd].ContourPlanes = new ContourPlane[nitems];

                for (uint i = 0; i < nitems; i++) {
                    theReturn[pd].ContourPlanes[i] = new ContourPlane();

                    Item item2 = sqi2.GetItem(i + 1); // Item start at #1

                    DataSet nestedds2 = item2.GetNestedDataSet();
                    // (3006,0050) DS [43.57636\65.52504\-10.0\46.043102\62.564945\-10.0\49.126537\60.714... # 398,48 ContourData
                    Tag tcontourdata = new Tag(0x3006, 0x0050);
                    DataElement contourdata = nestedds2.GetDataElement(tcontourdata);


                    ByteValue bv = contourdata.GetByteValue();
                    uint len = bv.GetLength();
                    byte[] encapsulated_stream = new byte[len];
                    bv.GetBuffer(encapsulated_stream, len);

                    using (System.IO.MemoryStream ms = new System.IO.MemoryStream(encapsulated_stream, 0, (int)len, false, true)) {

                        using (System.IO.StreamReader sr = new System.IO.StreamReader(ms)) {
                            string theBytes = sr.ReadToEnd();
                            string[] theDataElements = theBytes.Split(splitters);
                            theReturn[pd].ContourPlanes[i].Verteces = new DicomVertex[(int)(theDataElements.Length / 3)];
                            for (int k = 0; k < theDataElements.Length; k += 3) {
                                DicomVertex theVertex = new DicomVertex();
                                theVertex.X = float.Parse(theDataElements[k + 0]);
                                theVertex.Y = float.Parse(theDataElements[k + 1]);
                                theVertex.Z = float.Parse(theDataElements[k + 2]);
                                theReturn[pd].ContourPlanes[i].Verteces[(int)(k / 3)] = theVertex;
                            //    System.Console.WriteLine("Coordinate: ({0}, {1}, {2})", theFirst, theSecond, theThird);
                            }
                        }
                    }

                }
            }
            return theReturn;
        }


        static string GetStringValueFromTag(DataElement inDataElement) {
            char[] splitter = new char[1];
            splitter[0] = '\t';
            string theDataString = inDataElement.toString();
            string[] splits = theDataString.Split(splitter);
            return splits[splits.Length - 1];
        }
        static DicomVertex GetVertexValuesFromTag(DataElement inDataElement) {
            DicomVertex theVertex = new DicomVertex();
            char[] splitter = new char[2];
            splitter[0] = '\\';
            splitter[1] = '\t';
            string theDataString = inDataElement.toString();
            string[] splits = theDataString.Split(splitter);
            theVertex.X = float.Parse(splits[splits.Length-3]);
            theVertex.Y = float.Parse(splits[splits.Length - 2]);
            theVertex.Z = float.Parse(splits[splits.Length - 1]);
            return theVertex;
        }

        static void WriteOutAllDataSetTags(DataSet inDS, string inFileName) {

            CSharpDataSet theCDS = new CSharpDataSet(inDS);
            System.Console.WriteLine("Writing out " + inFileName + " Tags");
            while (!theCDS.IsAtEnd()) {
                DataElement de = theCDS.GetCurrent();
                string deString = de.toString();
                System.Console.WriteLine("Found tag! " + deString.Substring(0, (deString.Length > 100 ? 100 : deString.Length)));
                theCDS.Next();
            }

            System.Console.WriteLine("Done writing out " + inFileName + " Tags");
        }
    }
}
