﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace dcmtoblock {
    public class Image3DBlock {

        public short[] Data {
            get;
            set;
        }

        public int XSize {
            get;
            set;
        }

        public int YSize {
            get;
            set;
        }

        public int ZSize {
            get;
            set;
        }

        public double XSpacing { get; set; }

        public double YSpacing { get; set; }
        public double ZSpacing { get; set; }//assume this is constant through the scan, for now

        public string SeriesUID { get; set; }

        /// <summary>
        /// The planes that make up the data block; useful for the upper left coords, as well as spacing.
        /// Shouldn't be written out, hence why we're not using the entirely generated protobuf object
        /// </summary>
        public List<ImagePlane> Planes { get; set; }


        /// <summary>
        /// Given the list of planes that are read in from a directory, make the 3D datablock
        /// </summary>
        /// <param name="inPlanes"></param>
        public void SetupBlock(List<ImagePlane> inPlanes) {
            if (inPlanes.Count < 1) return;
            Planes = inPlanes;
            XSize = inPlanes[0].XSize;//assumes that the volume is the same throughout the image series
            YSize = inPlanes[0].YSize;
            ZSize = inPlanes.Count;
            Data = new short[XSize * YSize * ZSize];

            //sort the planes based on the UpperLeft.Z
            inPlanes.Sort(new ImagePlaneComparer());

            for (int z = 0; z < ZSize; z++) {
                System.Console.WriteLine(inPlanes[z].UpperLeft.Z.ToString("f2"));

                System.Buffer.BlockCopy(inPlanes[z].Data, 0, Data, 
                    z * XSize * YSize * sizeof(short), XSize * YSize * sizeof(short));
            }
            SeriesUID = inPlanes[0].SeriesUID;
            XSpacing = inPlanes[0].XSpacing;
            YSpacing = inPlanes[0].YSpacing;
            ZSpacing = inPlanes[0].ZSpacing;
        }

        public void WriteToDisk(string theDirectory) {
            
            using (FileStream fs = new FileStream(theDirectory + "\\ImageBlock." + 
                SeriesUID + ".block", FileMode.Create)){
                byte[] theBuffer = System.BitConverter.GetBytes(XSize);
                fs.Write(theBuffer, 0, theBuffer.Length);
                theBuffer = System.BitConverter.GetBytes(YSize);
                fs.Write(theBuffer, 0, theBuffer.Length);
                theBuffer = System.BitConverter.GetBytes(ZSize);
                fs.Write(theBuffer, 0, theBuffer.Length);

                theBuffer = System.BitConverter.GetBytes(XSpacing);
                fs.Write(theBuffer, 0, theBuffer.Length);
                theBuffer = System.BitConverter.GetBytes(YSpacing);
                fs.Write(theBuffer, 0, theBuffer.Length);
                theBuffer = System.BitConverter.GetBytes(ZSpacing);
                fs.Write(theBuffer, 0, theBuffer.Length);


                theBuffer = new byte[Data.Length * sizeof(short)];
                System.Buffer.BlockCopy(Data, 0, theBuffer, 0, Data.Length * sizeof(short));
                fs.Write(theBuffer, 0, theBuffer.Length);
            }
        }

    }
}
