﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace dcmtoblock {
    public class ImagePlaneComparer : IComparer<ImagePlane> {
        int IComparer<ImagePlane>.Compare(ImagePlane x, ImagePlane y) {
            return y.UpperLeft.Z.CompareTo(x.UpperLeft.Z);
        }
    }

    /// <summary>
    /// the plane that's returned by an individual dicom plane reader
    /// </summary>
    public class ImagePlane {
        public short[] Data {
            get;
            set;
        }

        public int XSize {
            get;
            set;
        }

        public int YSize {
            get;
            set;
        }

        public double XSpacing { get; set; }
        public double YSpacing { get; set; }
        public double ZSpacing { get; set; }

        public DicomVertex UpperLeft { get; set; }

        public string SeriesUID { get; set; }

        public bool Subsample(double inNewXYSpacing) {

            if (XSpacing != YSpacing) {
                return false;
            }

            int oldXSize = XSize;
            int oldYSize = YSize;
            double resizeFactor = XSpacing / inNewXYSpacing;
            int newXSize = (int)Math.Ceiling((double)XSize * resizeFactor);
            int newYSize = (int)Math.Ceiling((double)YSize * resizeFactor);

            double[,] old = new double[oldYSize, oldXSize];
            double[,] newArray = new double[newYSize, newXSize];

            //copy the data into the old array
            int y, x;
            for (y = 0; y < oldYSize; y++) {
                for (x = 0; x < oldXSize; x++) {
                    old[x, y] = (double)Data[y * oldXSize + x];
                }
            }

            spline2d.bilinearresamplecartesian(ref old, oldYSize, oldXSize, ref newArray, newYSize, newXSize);
            
            XSize = newXSize;
            YSize = newYSize;
            Data = new short[XSize*YSize];
            XSpacing = inNewXYSpacing;
            YSpacing = inNewXYSpacing;
            for (y = 0; y < newYSize; y++) {
                for (x = 0; x < newXSize; x++) {
                    Data[y * newXSize + x] = (short)newArray[x, y];
                }
            }

            return true;

        }
    }
}
