﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using gdcm;
using System.IO;

//uses gdcm to read in the contents of a directory and then transfer the results into a block of data.
//there may be routines to do this in C++, but it's just too damn painful to get gdcm to work in C++
//what with headers, libs, and dlls being all over the damn place.

namespace dcmtoblock
{
    class Program
    {
        static void Main(string[] args)
        {
            //enumerate all the files in a directory, given as the args
            if (args.Length > 0){
                string theDirectory = args[0];
                if (!System.IO.Directory.Exists(theDirectory))
                {
                    System.Console.WriteLine("Directory does not exist, please enter a directory name.");
                    return;
                }


                double theInterpValue = 0.75;
                if (args.Length > 1){
                    if (args[1] == "-i")
                        double.TryParse(args[2], out theInterpValue);
                    if (args[1] == "-h"){
                        System.Console.WriteLine("Flags:");
                        System.Console.WriteLine("-i: Interpolation value (default: 0.75, <= 0 for no interpolation)");
                        return;
                    }
                }
                       

                List<string> ctFiles = new List<string>();
                Contour[] theContours = null;
                foreach (string fileName in System.IO.Directory.GetFiles(theDirectory)) {
                    //first, check to see if it's a ct dicom file
                    //do this now by just checking to see if the filename begins with ct
                    //instead of getting the modality tag, which we'll need to do
                    //also, assume that everything is correct viz spacing
                    if (fileName.Contains("\\CT.") && fileName.EndsWith(".dcm")) {
                        ctFiles.Add(fileName);
                    }
                    if (fileName.Contains("\\RS.") && fileName.EndsWith(".dcm")) {
                        theContours = DicomReader.ReadRSStruct(fileName);
                    }
                }
                if (theContours == null) {
                    System.Console.WriteLine("Contour readin failed.");
                    return;
                }

                List<ImagePlane> thePlanes = new List<ImagePlane>();
                foreach (string ctFile in ctFiles) {
                    thePlanes.Add(DicomReader.ReadDicomImage(ctFile, theInterpValue));
                }

                Image3DBlock theBlock = new Image3DBlock();
                theBlock.SetupBlock(thePlanes);
                //and we now have a volume of CT data!  yay!
                //now, to get the RT data out and register it to that initial data.

                theBlock.WriteToDisk(theDirectory);
                theBlock.Data = null;
                GC.Collect();//garbage collect to nuke large allocations
                GC.WaitForPendingFinalizers();

                foreach (Contour c in theContours) {
                    if (c.ContourPlanes == null) continue; //empty mask
                    ContourMask3D theContourMask = c.Produce3DContourMask(theBlock);
                    theContourMask.WriteToDisk(theDirectory, theBlock.SeriesUID);
                    GC.Collect();//garbage collect to nuke large allocations
                    GC.WaitForPendingFinalizers();
                }
            }
        }
    }
}
